import logging
import urllib2
import urllib

from bn import absimport
import simplejson

log = logging.getLogger(__name__)

class RemoteAPIService(object):
    def __init__(self, url):
        self.url = url

    @staticmethod
    def create(flow, name, config=None):
        if config is None:
            config = RemoteAPIService.config(flow, name)
        return RemoteAPIService(config.url)

    @staticmethod
    def config(flow, name):
        from configconvert import handle_option_error, handle_section_error
        if not flow.config.option_group.has_key(name):
            raise handle_section_error(
                flow, 
                name, 
                "'%s.url'"%(
                    name
                )
            )
        from conversionkit import chainConverters, Conversion
        from stringconvert import unicodeToUnicode
        from stringconvert.email import listOfEmails, unicodeToEmail
        from recordconvert import toRecord

        to_unicode = unicodeToUnicode()
        converter = toRecord(
            converters = {
                'url': to_unicode,
            }
        )
        conversion = Conversion(flow.config.option_group[name]).perform(
            converter
        )
        if not conversion.successful:
            handle_option_error(conversion, name)
        else:
            flow.config[name] = conversion.result
        return flow.config[name]

    def start(self, flow, name):
        def api(fn, args=None):
            api_url = self.url+fn.replace('.', '/')
            return consume_service(api_url, fn, args)
        flow[name] = api

def consume_service(api_url, fn, args=None):
    log.info('API URL: %r', api_url)
    if args is None:
        fp = urllib2.urlopen(api_url)
    else:
        post_data = urllib.urlencode(
            dict(
                args=simplejson.dumps(args)
            )
        )
        fp = urllib2.urlopen(
            api_url, 
            post_data,
        )
    data = fp.read()
    fp.close()
    try:
        result = simplejson.loads(data)
    except ValueError:
        print 'Error:',  data
        raise
    return result


class LocalAPIService(object):
    def __init__(self, mod_path):
        self.mod_path = mod_path

    @staticmethod
    def create(flow, name, config=None):
        #if config is None:
        #    config = LocalAPIService.config(flow, name)
        return LocalAPIService(flow.config.app.package+'.ext.api.area')

    @staticmethod
    def config(flow, name):
        from configconvert import handle_option_error, handle_section_error
        if flow.config.option_group.has_key(name):
            raise Exception('Did not expect there to be configuration for %r'%name)
        return {}

    def start(self, flow, name):
        def api(fn, args=None):
            module, function = fn.split('.')
            mod = absimport(self.mod_path+'.'+module, from_=function)
            if args is not None:
                return mod(flow, args)
            else:
                return mod(flow)
        flow[name] = api

